/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Classe Letra
 *
 * Verso: 17/01/2001, 11:00
 */

#include "Evento.h"
#include "Letra.h"

/**
 *
 */
Letra::Letra(char *letra, TMemo *caixaLetra, TList *inicioLinhas)
{
    this->caixaLetra = caixaLetra;
    this->inicioLinhas = inicioLinhas;
    this->letra = letra;
    inicioLastSelecao = 0;
    caixaLetra->Lines->Text = letra;
    selecoes = new TList;
    selecoes->Clear();
    posicaoAtual = 0;
    tamanho = 0;
    indice = 0;
}


/**
 *
 */
Letra::~Letra()
{
    selecoes->Clear();
    delete selecoes;
    selecoes = NULL;
    delete letra;
    letra = NULL;
    delete inicioLinhas;
    inicioLinhas = NULL;
}


/**
 * Seleciona a letra em uma posio pr definida.
 */
void Letra::on()
{
    SelecaoLetra *temp;

    if (posicaoAtual >= 0 && posicaoAtual < tamanho)
    {
        caixaLetra->SetFocus();
        temp = (SelecaoLetra *) selecoes->Items[posicaoAtual];
        caixaLetra->SelStart = temp->inicio;
        caixaLetra->SelLength = temp->tamanho;

        posicaoAtual++;
    }
    else // jogar excecao para que o repositorio corrija o erro.
    {
        posicaoAtual = 0;
    }
}


/**
 * Altera a posio atual de seleo da letra para a do timestamp recebido.
 */
void Letra::setPosicaoAtual(long timestamp)
{
    int i = 0;

    if (timestamp == 0)
    {
        posicaoAtual = 0;
    }
    else
    {
        // Procura uma seleo com timestamp maior que o recebido.
        while (i < tamanho
               && ((SelecaoLetra *)selecoes->Items[i])->timestamp <= timestamp)
        {
            i++;
        }
        // Seta a posio atual para um timestamp anterior
        posicaoAtual = i - 1;
        i = 1;
        // Garante que a posio atual est num timestamp anterior.
        while (posicaoAtual > 0
               && ((SelecaoLetra *)selecoes->Items[posicaoAtual])
                  ->timestamp == timestamp)
        {
            posicaoAtual--;
            i++;
        }
        // Faz a seleo da letra deste timestamp.
        for ( ; i > 0; i--) {
            on();
        }
    }
}

/**
 * Insere um comando para seleo de letra.
 */
void Letra::inserirSelecao(long timestamp, int inicio, int tamanho)
{
    SelecaoLetra *selecao = new SelecaoLetra;
    SelecaoLetra *scroll = new SelecaoLetra;
    inicioLinha *ultimaLinha;

    ultimaLinha = getLastStartOfLine(inicio);

    if (inicio < inicioLastSelecao)
    {
        listaSelecaoClear(0);
        //ultimaLinha->didScroll = false;
    }

    if (!ultimaLinha->didScroll)
    {
        scroll->timestamp = timestamp;
        scroll->inicio = getPosScroll(ultimaLinha->comecoLinha);
        scroll->tamanho = 0;
        selecoes->Add(scroll);
        ultimaLinha->didScroll = true;
        this->tamanho++;
    }
    else {
        delete scroll;
    }

    selecao->timestamp = timestamp;
    selecao->inicio = inicio;
    selecao->tamanho = tamanho;

    inicioLastSelecao = inicio;

    selecoes->Add(selecao);
    this->tamanho++;
}

/**
 * Para ser chamada aps todos as selees terem sido inseridas. Este
 * mtodo vai inserir selees vazias que faro com que a letra aparea
 * sempre na posio correta da caixa de texto.
 */
void Letra::organizarSelecoes()
{
    int *linhas;
    int numeroLinhas = 1;
    int i = 0;
    int j = 0;

    // L a letra, contando o nmero de linhas
    while (letra[i] != '\0')
    {
        if (letra[i] == '\n')
        {
            numeroLinhas++;
        }
        i++;
    }
    // cria um array com o nmero de linhas
    linhas = new int[numeroLinhas+1];
    i = 0;
    /* L a letra mais uma vez, armazenando no array de linhas a posio inicial
     * de cada linha. */
    while (letra[i] != '\0')
    {
        if (letra[i] == '\n')
        {
            linhas[j] = i + 1;
            j++;
        }
        i++;
    }
    /* Percorre a lista de selees inserindo selees de tamanho zero duas
     * linhas abaixo de cada seleo em uma nova linha. */
    i = 0;
    j = 1;


    // ordena a seleo.
/*    if (selecoes->Count > 1)
    {
        //selecoes->Sort((TListSortCompare) compararSelecoesLetra);
        //selecoes->Pack();
    }*/
}


/**
 * Retorna a prxima lista de eventos da Letra.
 */
ListaEventos* Letra::getEventosOn()
{
    long timestampInicial;
    ListaEventos *retorno = new ListaEventos;
    ElementoListaEventos *evento;

    if (indice < tamanho)
    {
        timestampInicial = ((SelecaoLetra *)selecoes->Items[indice])->timestamp;
        retorno->timestamp = timestampInicial;
        retorno->eventos = new TList;
        retorno->eventos->Clear();
        // Percorre a lista incluindo todas as selees do mesmo timestamp.
        while (indice < tamanho && timestampInicial
                  == ((SelecaoLetra *)selecoes->Items[indice])->timestamp)
        {
            evento = new ElementoListaEventos;
            evento->evento = this;
            evento->tipo = ON;
            retorno->eventos->Add(evento);
            indice++;
        }
    }
    else
    {   // indica que chegou ao final do repositrio.
        retorno->timestamp = FIM_REPOSITORIO;
    }
    return retorno;
}

TList* Letra::getInicioLinhas()
{
    return this->inicioLinhas;
}

/*
* Recebe como argumento a posicao da selecao atual e retorna o struct q guarda
* a posicao do ultimo comeco de linha e o booleano q indica se esta linha
* jah realizou o scroll.
*/
inicioLinha* Letra::getLastStartOfLine(int posicaoAtual)
{
    inicioLinha *lastOne;
    int i;

    for(i=0; i < inicioLinhas->Count; i++)
    {
        if(((inicioLinha*)(inicioLinhas->Items[i]))->comecoLinha <= posicaoAtual)
        {
            lastOne = (inicioLinha *) inicioLinhas->Items[i];
        }
    }
    return lastOne;
}

/*
* Verifica se eh possivel pular as tres linhas do scroll.
* (inicioLinhas->Count equivale ao numero de linhas.)
* se n for possivel, faz o scroll para a ultima linha.
* (inicioLinhas->Count-1)
*/
int Letra::getPosScroll(int comecoLinhaAtual)
{
    int i,retorno = comecoLinhaAtual;
    for(i=0; i < inicioLinhas->Count; i++)
    {
        if(((inicioLinha*) (inicioLinhas->Items[i]))->comecoLinha == comecoLinhaAtual)
        {
            if (i < ((inicioLinhas->Count) - 3))
            {
                retorno = ((inicioLinha*) inicioLinhas->Items[i + 3])->comecoLinha;
            }
            else
            {
                retorno = ((inicioLinha*) inicioLinhas->Items[(inicioLinhas->Count -1)])->comecoLinha;
            }
        }
    }
    return retorno;
}

int Letra::getTipo()
{
    return LETRA;
}

/*
* Desmarca todos os 'didScroll' qd a selecao
* de letra volta pra um ponto anterior na musica.
*/
void Letra::listaSelecaoClear(int start)
{
    int i;

    for(i = start ; i < inicioLinhas->Count; i++)
    {
        ((inicioLinha *) inicioLinhas->Items[i])->didScroll = false;
    }
}


/**
 * Mtodo Auxiliar e Global.
 * Compara duas SelecaoLetra's,  utilizada para a ordenao da lista de
 * selees.
 */
int __fastcall compararSelecoesLetra(SelecaoLetra *selecao1, SelecaoLetra *selecao2)
{
    int retorno = 0;
    /*SelecaoLetra *sel1 = (SelecaoLetra *)selecao1;
    SelecaoLetra *sel2 = (SelecaoLetra *)selecao2;*/

    if (selecao1 == NULL)
    {
        retorno = 1;
    }
    else if (selecao2 == NULL)
    {
        retorno = -1;
    }
    else
    {
        // a prioridade  o timestamp.
        if (selecao1->timestamp > selecao2->timestamp)
        {
            retorno = 1;
        }
        else if (selecao1->timestamp < selecao2->timestamp)
        {
            retorno = -1;
        }
        else // caso os timestamps sejam iguais,  considerado menor algum que tenha
        {    // seleo de tamanho zero.
            if (selecao1->tamanho == 0)
            {
                retorno = -1;
            }
            else if (selecao2->tamanho == 0)
            {
                retorno = 1;
            }
        }
    }
    return retorno;
}