/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Definio da Classe Acorde
 *
 * Verso: 29/05/2001, 16:00
 */
#include "Acorde.h"
#include "SaidaMIDI.h"
#include "Nota.h"
#include "Braco.h"
#include <stdio.h>


// Guarda os nomes pra transposicao

/**
 *
 */
Acorde::Acorde(char *nome, Corda *cordas)
{
    this->nome = nome;
    this->cordas = cordas;
    this->calcularDedos();
}

/**
 *  Destrutor da classe
 */
Acorde::~Acorde() {

    delete[] this->nome;
    delete[] this->cordas;
    delete[] this->notas;

}


char* Acorde::getNome()
{
    return nome;
}


Corda* Acorde::getCordas()
{
    return cordas;
}

/**
 * Toca uma nota especfica do acorde de acordo com o dedo da mo direita.
 */
void Acorde::tocar(int dedoDireito, int volume)
{
/*
    #ifndef DEBUGANDO
    ArquivoLog::escrever("TocarAcorde\n");
    #endif
*/

     if (!eAcordeNulo()) {
        saidaMIDI->tocarNota(CANAL_VIOLAO, volume, notas[dedoDireito]);
        braco->acenderNotaAcorde(dedoDireito);
     }
}

/**
 * Pra uma nota especfica do acorde, de acordo com o dedo da mo direita.
 */
void Acorde::parar(int dedoDireito)
{
/*
    #ifndef DEBUGANDO
    ArquivoLog::escrever("PararAcorde\n");
    #endif
*/

     if (!eAcordeNulo()) {
        saidaMIDI->pararNota(CANAL_VIOLAO, notas[dedoDireito]);
        braco->apagarNotaAcorde(dedoDireito);
     }
}

/**
 * Calcula as notas (frequncia MIDI) do acorde e relaciona com os dedos da mo
 * direita.
 */
void Acorde::calcularDedos()
{
    notas = new int[7];
    for (int corda = PRIMEIRA_CORDA; corda <= SEXTA_CORDA; corda++)
    {
        notas[cordas[corda].dedoDireito]
            = calcularNota(corda, cordas[corda].casa);
    }
}

/**
 * Retona true se o acorde  um acorde "nulo" ou "em branco".
 */
bool Acorde::eAcordeNulo() {
    bool eNulo = true;

    for (int i = PRIMEIRA_CORDA; i <= SEXTA_CORDA && eNulo; i++) {
        if (cordas[i].dedoDireito != 6) {
            eNulo = false;
        }
    }
    return eNulo;
}

char* Acorde::filtraNome() {
char* retorno = new char[10];
int i;
    retorno[0] = this->nome[0];
    if(this->nome[1] == '#' || this->nome[1] == 'b') {
        retorno[1] = this->nome[1];
        retorno[2] = '\0';
    } else {
        retorno[1] = '\0';
    }
    return (retorno);
}

void Acorde::transporNome(int direcao, int numSemitons, bool bemol) {
char* inicioNome;
int indexNome, comecoRestoNome, indexBarra;
bool inversao;
AnsiString* nomeAuxiliar;

    // filtra soh o comeco do nome
    inicioNome = filtraNome();
    // verifica a qtd de caracteres do comeco
    if(inicioNome[1] == '\0') {
        comecoRestoNome = 1;
    } else {
        comecoRestoNome = 2;
    }

    indexNome = getIndexNome(inicioNome,bemol);
    indexNome = getNewIndex(direcao,numSemitons,indexNome);

    if ((inversao = calculaBemol(cabecalho->getTonalidade(),indexNome))) {
        bemol = inversao;
    }

    strcpy(inicioNome,getNomeIndex(indexNome,bemol));

    // Faz a transferencia das strings e fim-de-papo, caso n
    //haja inversao
    strcat(inicioNome,&nome[comecoRestoNome]);
    this->nome = new char[20];
    strcpy(this->nome,inicioNome);

    // Verifica se hah inversao
    if ((indexBarra = getLastBar()) != -1) {
        // e se houver um # com baixo em b, por exemplo? ....
        indexNome = getIndexNome(&nome[indexBarra + 1],bemol);
        // se for encontrado um correspondente para a string
        if(indexNome != -1) {
            // calcula o novo index
            indexNome = getNewIndex(direcao,numSemitons,indexNome);
            // copia para a string inicioNome a nova string
            strcpy(inicioNome,getNomeIndex(indexNome,bemol));
            strcpy(&nome[indexBarra + 1],inicioNome);
        }
    }
    nomeAuxiliar = new AnsiString(nome);
    strcpy(nome,nomeAuxiliar->Trim().c_str());

}

int Acorde::getIndexNome(char* inicioNome, bool usarArrayDeBemois) {
char bemois[12][3] =
                {"C","Db","D","Eb","E","F","Gb","G","Ab","A","Bb","B"};
char susten[12][3] =
                {"C","C#","D","D#","E","F","F#","G","G#","A","A#","B"};
int indexRetorno, i;


    indexRetorno = -1;

    for(i = 0; i < 13; i++) {
        if (usarArrayDeBemois) {
          if(strcmp(inicioNome,bemois[i]) == 0) {
              indexRetorno = i;
              break;
          }
        } else {
          if(strcmp(inicioNome,susten[i]) == 0) {
              indexRetorno = i;
              break;
          }
        }
    }
    return indexRetorno;
}

char* Acorde::getNomeIndex(int index, bool usarArrayDeBemois) {
char bemois[12][3] =
                {"C","Db","D","Eb","E","F","Gb","G","Ab","A","Bb","B"};

// Guarda ...
char* retorno = new char[5];
char susten[12][3] =
                {"C","C#","D","D#","E","F","F#","G","G#","A","A#","B"};

    if (index > 11 || index < 0) {
        Application->MessageBox("Erro na Busca do nome do acorde","Erro",0);
    }


    if(usarArrayDeBemois) {
        strcpy(retorno,bemois[index]);
    } else {
        strcpy(retorno,susten[index]);
    }
    return retorno;
}

// Retorna se um acorde eh ou nao bemol
bool Acorde::ehBemol() {
     return ((nome[1] != '\0') && (nome[1] == 'b'));
}

// Com o nome jah transposto, procura as novas cordas na base de acordes
void Acorde::ajustaCordas() {
FILE* base;
char nomeTemp[15], *caminhoBase;
int i;
Corda* cordaTemp;
char bufferLixo[100], aspas;
AnsiString filePath;
AnsiString* nomeErro;


    // Extrai o caminho do executavel
    filePath = ExtractFilePath(Application->ExeName);
    caminhoBase = new char[strlen(filePath.c_str())+30];
    strcpy(caminhoBase,filePath.c_str());
    strcat(caminhoBase,"base_de_acordes.txt");

    if((base = fopen(caminhoBase,"r"))==NULL) {
        Application->MessageBox("Nao foi possivel encontrar a base.","Transposicao de acordes",0);
        return;
    }
    delete caminhoBase;

    while (!feof(base)) {
        fscanf(base,"%c",&aspas);
        fscanf(base,"%[^\"]",nomeTemp);
        fscanf(base,"%c",&aspas);
        if(strcmp(nomeTemp,nome) == 0) {
            cordaTemp = new Corda[6];
            for (i = 5; i >= 0; i--) {
                fscanf(base,"%d %d %d\n",&cordaTemp[i].casa,
                                         &cordaTemp[i].dedoDireito,
                                         &cordaTemp[i].dedoEsquerdo);
            }
            this->cordas = cordaTemp;
            this->calcularDedos();
            fclose(base);
            return;
        } else {
            // pula os inteiros das cordas
            fscanf(base,"%[^\"]",bufferLixo);
        }
    }
    fclose(base);
    nomeErro = new AnsiString(nome);
    Application->MessageBox(("Nao encontrou o acorde"+(*nomeErro)).c_str(),
                                  "Erro",0);
    this->cordas = Constantes::cordasVazias;
    this->calcularDedos();
}

int Acorde::getLastBar() {
int retorno = -1;
int i;
    for(i = 0; nome[i] != '\0'; i++) {
        if(nome[i] == '/') {
            retorno = i;
        }
    }
    return (retorno);
}

int Acorde::getNewIndex(int direcao, int numSemitons, int indexAtual) {
int indexRetorno;

    // calculo do novo indice da transposicao
    switch (direcao)
    {
      case ACIMA:
        indexRetorno = ((indexAtual + numSemitons) % 12);
      break;
      case ABAIXO:
        if(indexAtual >= numSemitons) {
            indexRetorno = (indexAtual - numSemitons);
        } else {
            indexRetorno = (12 - (numSemitons - indexAtual));
        }
      break;
    }
    return (indexRetorno);
}

bool Acorde::calculaBemol(int tom, int nota) {
bool retorno = false;

    switch (tom)
    {
      // F
      case 7:
        // Bb
        if (nota == 10 || nota == 11) {
            retorno = true;
        }
      break;
      // Si bemol
      case 15:
        // Bb e Eb
        if (nota == 10 || nota == 11 || nota == 4) {
            retorno = true;
        }
      break;
      // Mi bemol
      case 5:
        // Bb Eb Ab
        if (nota == 10 || nota == 11 || nota == 4 || nota == 9 || nota == 8) {
            retorno = true;
        }
      break;
      // L bemol
      case 12:
        // Bb Eb Ab Db
        if (nota == 10 || nota == 11 || nota == 4 || nota == 9 || nota == 8
                       || nota == 1 || nota == 2) {
            retorno = true;
        }
      break;
      // R bemol
      case 2:
        // Bb Eb Ab Db Gb
        if (nota == 10 || nota == 11 || nota == 4 || nota == 9 || nota == 8
                       || nota == 1 || nota == 2 || nota == 6 || nota == 7) {
            retorno = true;
        }
      break;
      // Sol bemol
      case 9:
        // Bb Eb Ab Db Gb Cb
        if (nota == 10 || nota == 11 || nota == 4 || nota == 9 || nota == 8
                       || nota == 1 || nota == 2 || nota == 6 || nota == 7
                       || nota == 0) {
            retorno = true;
        }
      break;
    }
    return retorno;
}

