/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Classe RepositorioAcordes
 *
 * Verso: 29/05/2001, 16:00
 */
#include "RepositorioAcordes.h"
#include "Braco.h"
#include "TDacViolao.h"
//*#
#include <stdio.h>
//#*/

/**
 * Construtor da Classe
 */
RepositorioAcordes::RepositorioAcordes(SelecaoAcordeMemo** selecaoAcordeMemo)
{
    listaAcordes = new TList;
    listaAcordes->Clear();
    presencaBemois = new TList;

    eventos = new TList;
    eventos->Clear();
    indiceListaEventos = 0;
    tamanhoListaAcordes = 0;
    tamanhoListaEventos = 0;
    posicaoAtual = 0;
    this->selecaoAcordeMemo = (*selecaoAcordeMemo);
}


/**
 * Destrutor da Classe
 */
RepositorioAcordes::~RepositorioAcordes()
{
    this->limpar();

    delete listaAcordes;
    delete presencaBemois;
    
    listaAcordes = NULL;
    delete eventos;
    eventos = NULL;
}

/**
 * Retorna uma lista de acordes
 */
TList* RepositorioAcordes::getListaAcordes() {
    return listaAcordes;
}

/**
 * Retorna uma lista de acordes pronta para imprimir, ou seja, sem o acorde em
 * branco.
 */
TList* RepositorioAcordes::getListaAcordesParaImpressao() {

    TList *listaAcordesImpressao = new TList;
    listaAcordesImpressao->Clear();
    Acorde *acordeTeste;

    for (int i = 0; i < tamanhoListaAcordes; i++) {
        acordeTeste = (Acorde *)((ListaAcordes *)listaAcordes->Items[i])->acorde;
        if (!acordeTeste->eAcordeNulo()) {
            listaAcordesImpressao->Add(acordeTeste);
        }
    }
    return listaAcordesImpressao;
}

/**
 * Chama o "on" do acorde atual e atualiza a posio atual.
 */
void RepositorioAcordes::on()
{
static int ultimaPosicao = -1;

/*
    #ifndef DEBUGANDO
    ArquivoLog::escrever("AcordeOn\n");
    #endif
*/

    if (posicaoAtual >= 0 && posicaoAtual < tamanhoListaEventos)
    {
        acordeAtual = ((EventoAcorde *)eventos->Items[posicaoAtual])->acorde;
        if (braco->MostrarHarmonia
                // && posicaoAtual != ultimaPosicao
                ) {
            ultimaPosicao = posicaoAtual;
            braco->setAcorde(acordeAtual);
        }
        braco->atualizarLabelAcorde(acordeAtual->getNome());

        // seleciona o acorde no memo
        selecaoAcordeMemo->selecionar(posicaoAtual);

        posicaoAtual++;
        if (posicaoAtual < tamanhoListaEventos)
        {
            braco->setProximoAcorde(((EventoAcorde *)eventos->Items[posicaoAtual])->acorde->getNome());
        }
        else
        {
            braco->setProximoAcorde(" ");
        }
    }
    else // jogar excecao para que o repositorio corrija o erro.
    {
        posicaoAtual = 0;
        selecaoAcordeMemo->desfazerSelecao();
    }
}


/**
 * Altera a posio atual do Andamento para o do timestamp recebido.
 */
void RepositorioAcordes::setPosicaoAtual(long timestamp)
{
    int i = 0;

    if (timestamp == 0)
    {
        posicaoAtual = 0;
        selecaoAcordeMemo->setPosicaoAtual(NAO_SELECIONADO);
    }
    else
    {
        // Procura uma seleo com timestamp maior que o recebido.
        while (i < tamanhoListaEventos
               && ((EventoAcorde *)eventos->Items[i])->timestamp < timestamp)
        {
            i++;
        }
        // Seta a posio atual para um timestamp anterior
        posicaoAtual = i - 1;
        // Ativa o acorde deste timestamp.
        on();
    }
}


/**
 * Insere um acorde no repositrio, este acorde ser tratado sempre pelo indice
 * recebido.
 */
void RepositorioAcordes::inserir(Acorde *acorde, int indice)
{
    int tamanho;
    int i = 0;
    ListaAcordes *temp;
    bool* bemol = new bool;

    // verifica se um acorde eh bemol
    *bemol = acorde->ehBemol();
    presencaBemois->Add(bemol);

    /*#
    FILE *logFile;
    logFile = fopen("dvlog.log", "a");
    fprintf(logFile, "\t\tEntrou no mtodo inserir!\n");
    fclose(logFile);
    /#*/

    tamanho = listaAcordes->Count;
    /* Mexe com a lista de acordes */
    /* Verifica se o Acorde (indice) j existe. */
    while (i < tamanho)
    {   /* Caso a nota exista, seu timestamp  inserido na lista de eventos
         * do mesmo tipo */
        if (((ListaAcordes *)listaAcordes->Items[i])->indice == indice)
        {
            // caso j exista, joga exceo
            /*#
            logFile = fopen("dvlog.log", "a");
            fprintf(logFile, "Inserindo Acorde: acorde (%s) j existe!\n", acorde->getNome());
            fclose(logFile);
            /#*/
        }
        i++;
    }
    /*#
    logFile = fopen("dvlog.log", "a");
    fprintf(logFile, "\t\tAlocando memoria pra temp...\n");
    fclose(logFile);
    */
    temp = new ListaAcordes;
    /*#
    logFile = fopen("dvlog.log", "a");
    fprintf(logFile, "\t\tAlocou temp!\n");
    fclose(logFile);
    /*/
    temp->indice = indice;
    temp->acorde = acorde;
    tamanhoListaAcordes++;
    /*#
    logFile = fopen("dvlog.log", "a");
    fprintf(logFile, "\t\tAdicionando (%s) na lista...\n", acorde->getNome());
    fclose(logFile);
    */
    listaAcordes->Add(temp);
    /*#
    logFile = fopen("dvlog.log", "a");
    fprintf(logFile, "\t\tAdicionou (%s) na lista!\n", acorde->getNome());
    fclose(logFile);
    */
}


/**
 * Insere um evento no repositrio.
 */
void RepositorioAcordes::inserirEvento(int indice/*, tipoEvento*/, long timestamp) {

    EventoAcorde *temp = new EventoAcorde;
    bool achou = false;
    int i = 0;

    while (!achou && i < tamanhoListaAcordes) {

        if (((ListaAcordes *)listaAcordes->Items[i])->indice == indice) {
            temp->acorde = ((ListaAcordes *)listaAcordes->Items[i])->acorde;
            achou = true;
        }
        i++;
    }
    temp->timestamp = timestamp;
    eventos->Add(temp);
    tamanhoListaEventos++;
}

/**
 *
 */
void RepositorioAcordes::ordenar()
{
    if (eventos->Count > 1) {

//        eventos->Sort((TListSortCompare) compararEventoAcordes);
        eventos->Pack();
    }
}


/**
 *
 */
void RepositorioAcordes::limpar()
{
    int tamanho;
    int i;

    listaAcordes->Pack();
    tamanho = listaAcordes->Count;
    // Percorre a lista de acordes destruindo cada elemento.
    for (i = 0; i < tamanho; i++)
    {
        // deleta a nota.
        delete ((ListaAcordes *)listaAcordes->Items[i])->acorde;
    }
    // limpa as lista de notas e eventos.
    listaAcordes->Clear();
    eventos->Clear();
    // Zera o indice e o tamanho da lista de eventos.
    indiceListaEventos = 0;
    tamanhoListaEventos = 0;
    tamanhoListaAcordes = 0;
}


/**
 *
 */
void RepositorioAcordes::reset()
{
    indiceListaEventos = 0;
}


/**
 * Retorna os eventos Acorde::on do repositrio.
 */
ElementoEvento* RepositorioAcordes::getEventoOn()
{
    EventoAcorde *temp;
    ElementoEvento *retorno = new ElementoEvento;

    // Percorre toda a lista de eventos.
    if (indiceListaEventos < tamanhoListaEventos)
    {
        temp = (EventoAcorde *)eventos->Items[indiceListaEventos];
        retorno->evento = this;
        retorno->timestamp = temp->timestamp;
        retorno->tipo = ON;
        indiceListaEventos++;
    }
    else
    {   // indica que chegou ao final do repositrio.
        retorno->timestamp = FIM_REPOSITORIO;
    }
    return retorno;
}

/**
 * Toca uma nota especfica do acorde de acordo com o dedo da mo direita.
 */
void RepositorioAcordes::tocarAtual(int dedoDireito, int volume)
{
    if (braco->MostrarHarmonia) {
       this->acordeAtual->tocar(dedoDireito, volume);
    }
}

/**
 * Pra uma nota especfica do acorde, de acordo com o dedo da mo direita.
 */
void RepositorioAcordes::pararAtual(int dedoDireito)
{
    if (braco->MostrarHarmonia) {
        this->acordeAtual->parar(dedoDireito);
    }
}

/**
 * Retorna o tipo do evento correspondente. Mais utilizada para
 * determinar a prioridade de execuo de cada evento.
 */
int RepositorioAcordes::getTipo()
{
    return REPOSITORIO_ACORDES;
}
int RepositorioAcordes::getPosicaoAtual()
{
    return (this->posicaoAtual);
}

void RepositorioAcordes::setPosicaoAtual(int atual)
{
    this->posicaoAtual = atual;
}


/**
 * Mtodo Auxiliar e Global.
 * Compara dois EventoAcorde's,  utilizada para a ordenao da lista de
 * eventos.
 */
int __fastcall compararEventoAcordes(void *evento1, void *evento2)
{
    int retorno = 0;
    EventoAcorde *ev1 = (EventoAcorde *)evento1;
    EventoAcorde *ev2 = (EventoAcorde *)evento2;

    if (evento1 == NULL)
    {
        retorno = 1;
    }
    else if (evento2 == NULL)
    {
        retorno = -1;
    }
    else
    {
        // a prioridade  o timestamp.
        if (ev1->timestamp > ev2->timestamp)
        {
            retorno = 1;
        }
        else
        {
            retorno = -1;
        }
    }
    return retorno;
}

//---------------------------------------------------------------------------
void RepositorioAcordes::transpor(int direcao, int numSemitons)
{
int i;
Acorde* acordeTemp;
char* novoNome;

    transporTomDaMusica(direcao,numSemitons);
    for(i = 0; i < listaAcordes->Count; i++) {
        acordeTemp = ((Acorde*)((ListaAcordes*)listaAcordes->Items[i])->acorde);
        if (!acordeTemp->eAcordeNulo()) {
          acordeTemp->transporNome(direcao,numSemitons,(*(bool*)presencaBemois->Items[i]));
          acordeTemp->ajustaCordas();
          // ajusta a lista que indica se tem bemol ou nao...
          (*(bool*)presencaBemois->Items[i]) = acordeTemp->ehBemol();
        }
    }
}

//---------------------------------------------------------------------------
void RepositorioAcordes::setSelAcordes()
{
    selecaoAcordeMemo->setSelecoes();
}
//---------------------------------------------------------------------------
void RepositorioAcordes::transporTomDaMusica(int direcao, int numSemitons)
{
int tom;
    tom = cabecalho->getTonalidade();
    switch (direcao)
    {
      case ACIMA:
        while(numSemitons > 0) {
          if (tom == 1 || tom == 4 || tom == 8 || tom == 11 || tom == 14) {
            tom++;
          }
          tom++;
          numSemitons--;
        }
        tom = tom%17;
      break;
      case ABAIXO:
        while(numSemitons > 0) {
          if (tom == 2 || tom == 5 || tom == 9 || tom == 12 || tom == 15) {
            tom--;
          }
          tom--;
          numSemitons--;
        }
        if(tom < 0) {
          tom = 17 + tom;
        }
      break;
    }
    cabecalho->setTonalidade(tom);

}
