/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Definio da Classe RepositorioAcordes
 *
 * Verso: 29/05/2001, 12:00
 */
#ifndef REPOSITORIO_ACORDES_H
#define REPOSITORIO_ACORDES_H

#include "Evento.h"
#include "Constantes.h"
#include "Acorde.h"
#include "SelecaoAcordeMemo.h"

class RepositorioAcordes : public Evento
{
    public:

        /**
         *
         */
        RepositorioAcordes(SelecaoAcordeMemo** selecaoAcordeMemo);

        /**
         *
         */
        ~RepositorioAcordes();

        /**
         *
         */
        void on();

        /**
         *
         */
        void setPosicaoAtual(long timestamp);


        /**
         * Insere um acorde no repositrio. O indice recebido ser o ndice pelo
         * qual este acorde ser referenciado no restante do programa.
         */
        void inserir(Acorde *acorde, int indice);

        /**
         * Insere um acorde no repositrio. Este acorde ser procurado na base
         * por indiceBase, e tratado no restante do programa por indice.
         */
        //void inserir(int indiceBase, int indice);

        /**
         * Insere um evento para um acorde da base. O ndice recebido indica a
         * qual o acorde o evento ser aplicado.
         * tipoEvento: On, Off, Slide, Bend, etc...
         */
        void inserirEvento(int indice/*, int tipoEvento*/, long timestamp);

        /**
         * Retorna a prxima lista de eventos do repositrio.
         */
        ElementoEvento *getEventoOn();

        /**
         * Toca uma nota especfica do acorde de acordo com o dedo da mo direita.
         */
        void tocarAtual(int dedoDireito, int volume);

        /**
         * Pra uma nota especfica do acorde, de acordo com o dedo da mo direita.
         */
        void pararAtual(int dedoDireito);


        void ordenar();

        /**
         *
         */
        void limpar();

        /**
         *
         */
        void reset();

        /**
         *
         */
        int getTipo();

        /**
         *  Transpoe a lista de acordes
         */
        void transpor(int, int);

        /**
         *
         */
        //void alterarGrauDificuldade(...);

        TList *getListaAcordes();

         /**
         * Retorna uma lista de acordes pronta para imprimir, ou seja, sem o
         * acorde em branco.
         */
        TList *getListaAcordesParaImpressao();

        // Armazena as estruturas dos eventos: acordes e timestamps.
        TList *eventos;

        int getPosicaoAtual();
        void setPosicaoAtual(int);

        // Ajusta a selecao de acordes da cifra
        void setSelAcordes();

    private:
        // Armazena as estruturas com os acordes e seus respectivos indices.
        TList *listaAcordes;

        // Uma lista do mesmo tamanho q listaAcordes, q armazena a presenca
        //de bemol em cada acorde, pra ajudar na transposicao
        TList* presencaBemois;

        // Guarda o ndice atual da lista de eventos.
        int indiceListaEventos;

        // Guarda o tamanho da lista de eventos.
        int tamanhoListaEventos;

        // guarda o tamanho da lista de acordes.
        int tamanhoListaAcordes;

        // Guarda o ndice para a posio atual do acorde na lista de eventos.
        int posicaoAtual;

        // Guarda o acorde atual para poder ser tocado pelo ritmo.
        Acorde *acordeAtual;

        // guarda um ponteiro pro objeto do tipo SelecaoAcordeMemo
        SelecaoAcordeMemo* selecaoAcordeMemo;

        // Transpoe o tom da musica diretamente no cabecalho
        void transporTomDaMusica(int,int);

};

extern RepositorioAcordes *acordes;

/**
 * Mtodo Auxiliar e Global.
 * Compara dois EventoAcorde's,  utilizada para a ordenao da lista de
 * eventos.
 */
int __fastcall compararEventoAcordes(void *evento1, void *evento2);

#endif
