/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Definio da Classe SelecaoAcordeMemo
 *
 * Verso: 16/07/2001, 24:30
 */

#include "SelecaoAcordeMemo.h"
#include <stdlib.h>

#include <stdio.h>

// array que contem as posicoes em pixels das posicoes
int pixels[47] = {0,5,15,25,35,45,55,65,75,85,95,105,115,125,135,145,
                  155,165,175,185,195,205,215,225,235,245,255,265,275,285,295,
                  305,315,325,335,345,355,365,375,385,395,405,415,425,435,445};

/*
    Construtor
*/
SelecaoAcordeMemo::SelecaoAcordeMemo(TMemo* caixaLetra, Letra* letra) {

    this->caixaLetra = caixaLetra;
    this->listaSelecoes = new TList;
    this->selecaoAtual = NAO_SELECIONADO;
    this->selecaoAtual = 0;
    this->letra = letra;
    this->canvasMemo = new TCanvas();
    this->rectSelecao.Top = 1;
    this->rectSelecao.Bottom = 18;
    podeRedesenhar = false;
    this->inicioSel = 0;

}

/*
    Destrutor
*/
SelecaoAcordeMemo::~SelecaoAcordeMemo() {
    int i;
    int tamanho = this->listaSelecoes->Count;

    // percorre a lista de selecoes deletando os eventos
    for(i=0; i < tamanho; i++) {

        if (this->listaSelecoes->Items[i] != NULL) {
            delete ((PosicaoAcordeMemo*)this->listaSelecoes->Items[i]);

        }
    }

    delete this->listaSelecoes;
}

/*
    Insere uma nova selecao, da as coordenadas iniciais e finais
*/
void SelecaoAcordeMemo::inserirSelecao(int inicio, int fim) {
    PosicaoAcordeMemo* novaSelecao = new PosicaoAcordeMemo;

    novaSelecao->inicio = inicio;
    novaSelecao->fim = fim;

    listaSelecoes->Add(novaSelecao);

}

/*
    Insere uma nova selecao, dada uma nova posicao
*/
void SelecaoAcordeMemo::inserirSelecao(PosicaoAcordeMemo* novaSelecao) {
    listaSelecoes->Add(novaSelecao);

}


/*
    Seleciona o acorde desejado no memo
*/
void SelecaoAcordeMemo::selecionar(int numeroAcorde) {
    int inicioDesenho,fimDesenho;
    int linhaAtual,proximaLinha, selecaoAntiga;
    PosicaoAcordeMemo* posicaoNova = (PosicaoAcordeMemo*)
                                        listaSelecoes->Items[numeroAcorde];
    // garante que a selecao de acorde sempre vem antes de uma selecao de letra
    /*
    if (posicaoNova->inicio > caixaLetra->SelStart) {
       caixaLetra->SelLength = 0;
       caixaLetra->SelStart = posicaoNova->inicio;
    }
    */

    // "diz" a classe letra onde esta a selecao de acorde
    letra->setInicioSelAcorde(posicaoNova->inicio);

    // calcula a linha atual e a proxima linha
    linhaAtual = letra->getLastLine(inicioSel);
    proximaLinha = letra->getLastLine(posicaoNova->inicio);
    selecaoAntiga = selecaoAtual;
    // desfaz a selecao antiga
    desfazerSelecao();


    // atualiza as posicoes atuais de inicio da selecao
    inicioSel = posicaoNova->inicio;
    fimSel = posicaoNova->fim;


    // calcula a posicao a ser desenhada
    inicioDesenho = inicioSel - letra->getLastStartOfLine(inicioSel);
    fimDesenho = inicioDesenho + (fimSel - inicioSel);


    // testa se ha necessidade de scroll
    while ((linhaAtual - proximaLinha) < -1 ||
           (linhaAtual - proximaLinha) > 1) {
         if ((selecaoAntiga + 1) <= numeroAcorde) {
            scrollLetra(3);
            linhaAtual+=3;
         } else {
            if ((selecaoAntiga - 1) >= numeroAcorde) {
                scrollLetra(-3);
                linhaAtual-=3;
            }
         }

    }

    // desenha a selecao
    desenharSelecao(inicioDesenho, fimDesenho);

    selecaoAtual = numeroAcorde;

}

/*
    Desfaz a ultima selecao e seta o acorde atual pra nulo
*/
void SelecaoAcordeMemo::desfazerSelecao() {

    if (selecaoAtual != NAO_SELECIONADO) {

        apagarSelecao();
        //selecaoAtual = NAO_SELECIONADO;
        
    }

}

void SelecaoAcordeMemo::apagarSelecao() {
     canvasMemo->Pen->Color = caixaLetra->Color;
     canvasMemo->Brush->Color = caixaLetra->Color;
     canvasMemo->Handle = GetDC(caixaLetra->Handle);
     canvasMemo->FrameRect(rectSelecao);

     ReleaseDC(0,canvasMemo->Handle);
     podeRedesenhar = false;

}

void SelecaoAcordeMemo::desenharSelecao(int inicio, int fim) {

     canvasMemo->Pen->Color = clLime;
     canvasMemo->Brush->Color = clLime;
     canvasMemo->Handle = GetDC(caixaLetra->Handle);

     rectSelecao.Left = pixels[inicio];
     rectSelecao.Right = pixels[fim+2];
     canvasMemo->FrameRect(rectSelecao);

     ReleaseDC(0,canvasMemo->Handle);
     podeRedesenhar = true;

}

void SelecaoAcordeMemo::scrollLetra(int numLinhas) {
int cont;

    /*
        Repetindo 3 vezes pra tentar ganhar eficiencia
    */

    if (numLinhas > 0) {
//        for (cont = 0; cont < numLinhas; cont++) {

            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEDOWN, 0);
            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEDOWN, 0);
            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEDOWN, 0);
  //      }
    } else {
//        for (cont = 0; cont < (-numLinhas); cont++) {
            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEUP, 0);
            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEUP, 0);
            SNDMSG(caixaLetra->Handle, WM_VSCROLL, SB_LINEUP, 0);
  //      }

    }

}

void SelecaoAcordeMemo::redesenhar() {

     if (podeRedesenhar) {
          canvasMemo->Pen->Color = clLime;
          canvasMemo->Brush->Color = clLime;
          canvasMemo->Handle = GetDC(caixaLetra->Handle);
          canvasMemo->FrameRect(rectSelecao);

          ReleaseDC(0,canvasMemo->Handle);

     }

}

// retorna o inicio da selecao atual
int SelecaoAcordeMemo::getInicioSel() {
    return inicioSel;

}


void SelecaoAcordeMemo::reset() {
    inicioSel = 0;
    selecaoAtual = NAO_SELECIONADO;

}


void SelecaoAcordeMemo::setPosicaoAtual(int posicao) {
/*
     if (posicao == NAO_SELECIONADO) {
        inicioSel = 0;
        fimSel = 0;
     } else {
*/
     if (posicao != NAO_SELECIONADO) {
        selecaoAtual = posicao;

        PosicaoAcordeMemo* posicaoNova = (PosicaoAcordeMemo*)
                                        listaSelecoes->Items[posicao];

        inicioSel = posicaoNova->inicio;
        fimSel = posicaoNova->fim;

     }
}

void SelecaoAcordeMemo::setSelecoes() {
// shift eh necessario por causa do /r/n
int i = 0, cont, cont2, shift=0;
bool achouColchetes = false;
bool ehLinhaAcorde = true;
bool linhaVazia = true;
bool achouBarraErre = false;
TList* trechoRepetido;
PosicaoAcordeMemo* posicaoTemp;
char* cifra;
int tamanho;

    /*
        Comeca a varrer a letra para identificar acordes e suas repeticoes
        Observacoes:
        1 - As repeticoes sao separadas por colchetes. ex.  [Am G F]2x
        2 - Apos o ], segue um inteiro dizendo a quantidade de vezes que este
        trecho ira se repetir, seguido de x.
    */

    clear();

    cifra = letra->getLetra();
    tamanho = letra->getTamLetra();

    while (i < tamanho) {

        switch(cifra[i]) {
            case '[':
                linhaVazia = false;
                if (ehLinhaAcorde) {
                    achouColchetes = true;
                    trechoRepetido = new TList;
                }
            break;

            case ']':
                if (ehLinhaAcorde) {
                    achouColchetes = false;
                    char numeroRepeticoes[10];
                    int index = 0;
                    int numRepeticoes;
                    i++;
                    // simplesmente le a quantidade de vezes que vai repetir
                    while (cifra[i] != 'x') {
                        if (cifra[i] >= '0' && cifra[i] <= '9') {
                            numeroRepeticoes[index++] = cifra[i];
                        }
                        i++;
                    }
                    numeroRepeticoes[index] = '\0';
                    // terminou de ler esta quantidade
                    numRepeticoes = atoi(numeroRepeticoes);

                    // insere os acordes repetidos de novo
                    for (cont = 1; cont < numRepeticoes; cont++) {
                        for (cont2 = 0; cont2 < trechoRepetido->Count; cont2++) {
                            inserirSelecao(
                                (PosicaoAcordeMemo*)trechoRepetido->Items[cont2]);

                        }

                    }
                }
            break;

            case '\n':
            if(!linhaVazia){
                ehLinhaAcorde = !ehLinhaAcorde;
                linhaVazia = true;
            }
            if (!achouBarraErre) {
                shift++;
            }
            achouBarraErre = false;
            break;

            case ' ':
            break;

            case '\r':
            achouBarraErre = true;
            break;

            default:
                linhaVazia = false;

                if (ehLinhaAcorde) {
                    int inicio = i + shift;
                    int fim;

                    while (cifra[i] != ' ' && cifra[i] != '\n' && cifra[i]!='['
                                        && cifra[i] != ']' && cifra[i] !='\r'
                                        && i < tamanho) {
                        i++;
                    }
                    if (i == tamanho) {
                      break;
                    }
                    fim = i + shift - 1;
                    inserirSelecao(inicio, fim);
                    if (achouColchetes) {
                        posicaoTemp = new PosicaoAcordeMemo;
                        posicaoTemp->inicio = inicio;
                        posicaoTemp->fim = fim;
                        trechoRepetido->Add(posicaoTemp);

                    }
                    i--;
                }
            break;

        }
        i++;
    }
}

void SelecaoAcordeMemo::clear() {
int i;

    for(i = 0; i < listaSelecoes->Count; i++) {
        delete (listaSelecoes->Items[i]);
    }
    listaSelecoes->Clear();
    delete (listaSelecoes);
    listaSelecoes = new TList;
    this->selecaoAtual = NAO_SELECIONADO;
}




