/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Classe Nota
 *
 * Verso: 08/05/2001, 11:00
 */
#include "Nota.h"
#include "SaidaMIDI.h"

/**
 * Construtor da Classe.
 *
 * Parmetros:
 *      canal: canal onde a nota vai ser tocada (deve estar entre 0 e 15).
 *      volume: volume no qual a nota ser tocada (deve estar entre 0 e 127).
 *      nota: frequncia MIDI da nota a ser tocada.
 *      saidaMIDI: referncia da placa de som.
 */
Nota::Nota(int canal, int volume, int nota)
{
    this->canal = canal;
    this->volume = volume;
    this->nota = nota;
}


/**
 * Destrutor da Classe.
 */
Nota::~Nota() { }

/**
 * Toca a nota j definida.
 */
void Nota::on()
{
/*
    #ifndef DEBUGANDO
    ArquivoLog::escrever("NotaOn\n");
    #endif
*/
    saidaMIDI->tocarNota(canal, volume, nota);
}


/**
 * Para de tocar a nota.
 */
void Nota::off()
{
/*
    ArquivoLog::escrever("NotaOff\n");
*/
    saidaMIDI->pararNota(canal, nota);
}


/**
 * Compara duas notas.
 */
bool Nota::igual(Nota *nota)
{
    return (this->getTipo() == nota->getTipo() && this->canal == nota->canal &&
            this->volume == nota->volume && this->nota == nota->nota);
}


/**
 *
 */
int Nota::getTipo()
{
    return NOTA;
}

/**
 * Mtodo Auxiliar e global.
 * Compara dois EventoNota's,  utilizada para a ordenao da lista de
 * eventos.
 */
int __fastcall compararEventosNota(EventoNota *evento1, EventoNota *evento2)
{
    int retorno = 0;

    if (evento1 == NULL)
    {
        retorno = 1;
    }
    else if (evento2 == NULL)
    {
        retorno = -1;
    }
    else
    {    // a prioridade  o timestamp.
        if (evento1->timestamp > evento2->timestamp)
        {
            retorno = 1;
        }
        else
        {
            retorno = -1;
        }
    }
    return retorno;
}


/**
 * Mtodo Auxiliar e global.
 * Retorna a frequencia da nota equivalente a corda solta.
 */
int getNota(int corda)
{
    int retorno = 0;

    switch (corda)
    {
        case 5:
            retorno = 40;
            break;
        case 4:
            retorno = 45;
            break;
        case 3:
            retorno = 50;
            break;
        case 2:
            retorno = 55;
            break;
        case 1:
            retorno = 59;
            break;
        case 0:
            retorno = 64;
            break;
        default:
            retorno = retorno;
            break;
    }
    return retorno;
}

/**
 * Calcula a nota a partir da casa e da corda recebidos
 */
int calcularNota(int corda, int casa)
{
        if (casa >= 12) {
                casa = casa;
        }

    return (getNota(corda) + casa);
}

/**
 * Retorna a frequencia da nota
 */
int Nota::getFrequencia() {
    return this->nota;

}

/**
 * Retorna o canal da nota
 */
int Nota::getCanal() {
    return this->canal;

}

void Nota::setNota(int novaNota) {
     this->nota = novaNota;
}

int Nota::getNota() {
    return this->nota;
}

