/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Definio da Classe Nota
 *
 * Verso: 11/05/2001, 17:00
 */
#ifndef NOTA_H
#define NOTA_H

#include "evento.h"
#include "Constantes.h"

class Nota : public Evento
{
    public:

        /**
         * Construtor da Classe.
         *
         * Parmetros:
         *      canal: canal onde a nota vai ser tocada (deve estar entre 0 e
         *             15).
         *      volume: volume no qual a nota ser tocada (deve estar entre 0 e
         *              127).
         *      nota: frequncia MIDI da nota a ser tocada.
         */
        Nota(int canal, int volume, int nota);

        /**
         * Destrutor da Classe.
         */
        ~Nota();

        /**
         * Toca a nota j definida.
         */
        virtual void on();

        /**
         * Para de tocar a nota.
         */
        virtual void off();

        /**
         * Compara duas notas.
         */
        virtual bool igual(Nota *nota);

        /**
         *
         */
        virtual int getTipo();

        /**
         * Retorna a frequencia da nota
         */
        int getFrequencia();

        /**
         * Retorna o canal da nota
         */
        int getCanal();

        // Seta a frequencia da nota (para a transposicao)
        void setNota(int);

        // Retorna a frequencia da nota
        int getNota();


    private:

        // canal onde a nota vai ser tocada (deve estar entre 0 e 15).
        int volume;

        // volume no qual a nota ser tocada (deve estar entre 0 e 127).
        int canal;

        // frequncia MIDI da nota a ser tocada.
        int nota;
};

/**
 * Mtodo Auxiliar e global.
 * Compara dois EventoNota's,  utilizada para a ordenao da lista de
 * eventos.
 */
int __fastcall compararEventosNota(EventoNota *evento1, EventoNota *evento2);

/**
 * Mtodo Auxiliar e global.
 * Retorna a frequencia da nota equivalente a corda solta.
 */
int getNota(int corda);

/**
 * Calcula a nota a partir da casa e da corda recebidos
 */
int calcularNota(int corda, int casa);

#endif