/**
 * D'accord Music Software
 * http://www.daccord.com.br
 *
 * D'accord Violo
 * Classe RepositorioRitmos
 *
 * Verso: 29/05/2001, 12:00
 */
#include "RepositorioRitmos.h"
#include "Cabecalho.h"

/**
 * Construtor da Classe
 */
RepositorioRitmos::RepositorioRitmos()
{
    ritmos = new TList;
    ritmos->Clear();
    eventos = new TList;
    eventos->Clear();
    indiceListaEventos = 0;
    tamanhoListaRitmos = 0;
    tamanhoListaEventos = 0;
    this->eventosOn = new TList;
    this->eventosOn->Clear();
    this->eventosOff = new TList;
    this->eventosOff->Clear();
    indiceOn = 0;
    indiceOff = 0;
    tamanhoOn = 0;
    tamanhoOff = 0;
}

/**
 * Destrutor da Classe
 */
RepositorioRitmos::~RepositorioRitmos()
{
    this->limpar();
    delete ritmos;
    ritmos = NULL;
    delete eventos;
    eventos = NULL;
    delete eventosOn;
    delete eventosOff;
    eventosOn = NULL;
    eventosOff = NULL;
}

/**
 * Insere um ritmo no repositrio, este ritmo ser tratado sempre pelo indice
 * recebido.
 */
void RepositorioRitmos::inserir(Ritmo *ritmo, int indice)
{
    int tamanho;
    int i = 0;
    ListaRitmos *temp;

    tamanho = ritmos->Count;
    /* Mexe com a lista de ritmos */
    /* Verifica se o Ritmo (indice) j existe. */
    while (i < tamanho)
    {   /* Caso a nota exista, seu timestamp  inserido na lista de eventos
         * do mesmo tipo */
        if (((ListaRitmos *)ritmos->Items[i])->indice == indice)
        {
            // caso j exista, joga exceo
        }
        i++;
    }
    temp = new ListaRitmos;
    temp->indice = indice;
    temp->ritmo = ritmo;
    tamanhoListaRitmos++;
    ritmos->Add(temp);
}

/**
 * Insere um evento no repositrio.
 */
void RepositorioRitmos::inserirEvento(int indice/*, tipoEvento*/, long timestamp) {

    EventoRitmo *temp = new EventoRitmo;
    bool achou = false;
    int i = 0;

    while (!achou && i < tamanhoListaRitmos) {

        if (((ListaRitmos *)ritmos->Items[i])->indice == indice) {
            temp->ritmo = ((ListaRitmos *)ritmos->Items[i])->ritmo;
            achou = true;
        }
        i++;
    }
    if (!achou) {
        // tratamento de erros - nao achou o ritmo!
    }
    temp->timestamp = timestamp;
    eventos->Add(temp);
    tamanhoListaEventos++;
}
  

/**
 *
 */
void RepositorioRitmos::ordenar()
{
    if (eventos->Count > 1) {

//        eventos->Sort((TListSortCompare) compararEventoRitmos);
        eventos->Pack();
    }
}

/**
 *
 */
void RepositorioRitmos::limpar()
{
    int tamanho;
    int i;

    ritmos->Pack();
    tamanho = ritmos->Count;
    // Percorre a lista de ritmos destruindo cada elemento.
    for (i = 0; i < tamanho; i++)
    {
        // deleta a nota.
        delete ((Ritmo *)((ListaRitmos *)ritmos->Items[i])->ritmo);
    }
    // limpa as lista de notas e eventos.
    ritmos->Clear();
    eventos->Clear();
    this->eventosOn->Clear();
    this->eventosOff->Clear();
    // Zera o indice e o tamanho da lista de eventos.
    indiceListaEventos = 0;
    tamanhoListaEventos = 0;
    tamanhoListaRitmos = 0;
}

/**
 *
 */
void RepositorioRitmos::reset()
{
    indiceListaEventos = 0;
}

/**
 *
 */
ListaEventos* RepositorioRitmos::getEventosOn() {

    ListaEventos *retorno = new ListaEventos;

    if (indiceOn < tamanhoOn)
    {
        retorno = (ListaEventos *)eventosOn->Items[indiceOn];
        indiceOn++;
    }
    else
    {
        retorno->timestamp = FIM_REPOSITORIO;
    }
    return retorno;
}

/**
 *
 */
ListaEventos* RepositorioRitmos::getEventosOff() {

    ListaEventos *retorno = new ListaEventos;

    if (indiceOff < tamanhoOff)
    {
        retorno = (ListaEventos *)eventosOff->Items[indiceOff];
        indiceOff++;
    }
    else
    {
        retorno->timestamp = FIM_REPOSITORIO;
    }
    return retorno;
}

/**
 *
 */
void RepositorioRitmos::montarRepositorio() {

    int indice = 0;
    int tamanho = 0;
    long timestampInicial = 0;
    long timestampFinal = 0;
    EventoRitmo *temp;

    tamanho = tamanhoListaEventos - 1;
    while (indice < tamanho) {
        temp = (EventoRitmo *)eventos->Items[indice];
        timestampInicial = temp->timestamp;
        indice++;
        timestampFinal = ((EventoRitmo *)eventos->Items[indice])->timestamp;
        adicionarListaOn(temp->ritmo->getEventosOn(timestampInicial, timestampFinal));
        adicionarListaOff(temp->ritmo->getEventosOff(timestampInicial, timestampFinal));
    }
    if (indice == tamanho) {
        temp = (EventoRitmo *)eventos->Items[indice];
        timestampInicial = temp->timestamp;
        timestampFinal = cabecalho->getTempoTotal();
        adicionarListaOn(temp->ritmo->getEventosOn(timestampInicial, timestampFinal));
        adicionarListaOff(temp->ritmo->getEventosOff(timestampInicial, timestampFinal));
    }
}

/**
 *
 */
void RepositorioRitmos::adicionarListaOn(TList *eventosOn) {

    int tamanho = eventosOn->Count;
    int i;

    for (i = 0; i < tamanho; i++) {
        this->eventosOn->Add(eventosOn->Items[i]);
        this->tamanhoOn++;
    }
}

/**
 *
 */
void RepositorioRitmos::adicionarListaOff(TList *eventosOff) {

    int tamanho = eventosOff->Count;
    int i;

    for (i = 0; i < tamanho; i++) {
        this->eventosOff->Add(eventosOff->Items[i]);
        this->tamanhoOff++;
    }
}